;===========================================================================================
; Text_3.asm  Fasm Flat assembler
; Fonte et Texte en couleur
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

; Macro couleur dans eax
macro RGB red,green,blue
{   mov eax,0
    mov ah,byte blue
    shl eax,8
    mov ah,byte green
    mov al,byte red   }

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne
    ; ----------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur
    ; -----------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Obtenir la hauteur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CYSCREEN
    sub eax,[WinH]
    shr eax,1
    mov [WinY],eax

    ; Valider le choix de la structure WNDCLASS
    ; -----------------------------------------
    invoke RegisterClass,wc

    ; Cration de la fentre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,WS_EX_CLIENTEDGE,ClassName,Title,WS_OVERLAPPEDWINDOW+WS_VISIBLE,\
                          [WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affichage de la fentre
    ; -----------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT

    ; Rafraichissement de la fentre
    ; ------------------------------
    invoke UpdateWindow,[hWnd]

 ;----------------------------------
 ; Boucle de traitement des messages
 ;----------------------------------
 MessageLoop:
    ; Rceptionne les messages
    ; ------------------------
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram

    ; Traduit les touches d'entre clavier en messages
    ; ------------------------------------------------
    invoke TranslateMessage,msg

    ; Expdition des messages  la procdure Windows
    ; ---------------------------------------------
    invoke DispatchMessage,msg

    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande la fin de l'application
        ; -------------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_PAINT
        ; Dbut criture/dessin
        ; ---------------------
        invoke BeginPaint,[@hWnd],ps

        ; Cration d'une fonte de caractres
        ; ----------------------------------
        invoke CreateFont,24,16,0,0,400,0,0,0,OEM_CHARSET,OUT_DEFAULT_PRECIS,\
                          CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SCRIPT,NFont

        ; Choisit un objet suivant le contexte de dispositif hdc
        ; ------------------------------------------------------
        invoke SelectObject,[ps.hdc],eax
        mov [hFont],eax

        ; Fixer la couleur du texte
        ; -------------------------
        RGB 000,000,255
        invoke SetTextColor,[ps.hdc],eax

        ; Fixer la couleur de fond du texte
        ; ---------------------------------
        RGB 255,000,000
        invoke SetBkColor,[ps.hdc],eax

        ; Obtenir les coordonnes relatives de la fentre
        ; -----------------------------------------------
        invoke GetClientRect,[@hWnd],rect

        ; Ecriture dans la fentre
        ; ------------------------
        invoke DrawText,[ps.hdc],TxtString,-1,rect,DT_SINGLELINE+DT_CENTER+DT_VCENTER

        ; Choisit un objet suivant le contexte de dispositif hdc
        ; ------------------------------------------------------
        invoke SelectObject,[ps.hdc],[hFont]

        ; Fin criture/dessin
        ; -------------------
        invoke EndPaint,[@hWnd],eax

    .else
        ; Traitement par dfaut des autres messages Windows
        ; ------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
; ---------------------------------------------------------------------------------------
section '.idata' data readable writeable

  Title     db ' Fonte et Texte en couleur ',0
  ClassName db ' AsmGges Win32',0
  MsgEnd    db ' AsmGges France @2006  ',0
  TxtString db ' Fonte et Texte couleur ',0
  NFont     db 'script',0
  align 4

  WinX      dd 0
  WinY      dd 0
  WinW      dd 512
  WinH      dd 352

  hWnd      dd ?
  hFont     dd ?

  msg       MSG
  wc        WNDCLASS  0,WindowProc,0,0,NULL,NULL,NULL,COLOR_WINDOW+1,NULL,ClassName
  ps        PAINTSTRUCT
  rect      RECT

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================
